#include "../stdafx.h"
#include "DllInput.h"
#include "../MainHelper.h"
#include "../Resource.h"
#include "CommonFunc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDllInput::~CDllInput()
{
	delete internalIn;
	if (hModIn != NULL)
	{
		FreeLibrary(hModIn);
	}
	hModIn = NULL;
}

CDllInput::CDllInput(CString dllPath, HWND hWnd) : InputInterface(dllPath)
{
	internalIn = new CStandardInput("", hWnd);
	internalInMod = internalIn->GetIn_Module();
	pfun2 = NULL;
	pfun3 = NULL;

	if (dllPath.GetLength() != 0)
	{
		// returns a handle to the dll, otherwise NULL
		hModIn = LoadLibrary(dllPath); 
		if (hModIn == NULL)
		{
			AfxMessageBox(m_HelperIn.MyLoadString(IDS_FAILED_LOAD_INPUT) + GetFileName(), MB_OK | MB_ICONEXCLAMATION);
			dllInMod = internalInMod;
			SetFilePath("STANDARD");
			hModIn = NULL;
		}
		else
		{
			// returns the address of the dll function, otherwise NULL
			try
			{
				pfun1 = (winampGetInModule)GetProcAddress(hModIn, "winampGetInModule2");
				pfun2 = (LPwinampGetExtendedFileInfo)GetProcAddress(hModIn, "winampGetExtendedFileInfo");
				pfun3 = (LPwinampGetExtendedFileInfoW)GetProcAddress(hModIn, "winampGetExtendedFileInfoW");
				// (Dll function address) (function parameters)
				dllInMod = (pfun1)();

				//Init des fonctions non dclare par celle par dfaut..
				dllInMod->hMainWindow = hWnd;
				if (dllInMod->Config == NULL)
					dllInMod->Config = internalInMod->Config;
				if (dllInMod->About == NULL)
					dllInMod->About = internalInMod->About;
				if (dllInMod->Init == NULL)
					dllInMod->Init = internalInMod->Init;
				if (dllInMod->Quit == NULL)
					dllInMod->Quit = internalInMod->Quit;
				if (dllInMod->GetFileInfo == NULL)
					dllInMod->GetFileInfo = internalInMod->GetFileInfo;
				if (dllInMod->InfoBox == NULL)
					dllInMod->InfoBox = internalInMod->InfoBox;
				if (dllInMod->IsOurFile == NULL)
					dllInMod->IsOurFile = internalInMod->IsOurFile;
				if (dllInMod->Play == NULL)
					dllInMod->Play = internalInMod->Play;
				if (dllInMod->Pause == NULL)
					dllInMod->Pause = internalInMod->Pause;
				if (dllInMod->UnPause == NULL)
					dllInMod->UnPause = internalInMod->UnPause;
				if (dllInMod->IsPaused == NULL)
					dllInMod->IsPaused = internalInMod->IsPaused;
				if (dllInMod->Stop == NULL)
					dllInMod->Stop = internalInMod->Stop;
				if (dllInMod->GetLength == NULL)
					dllInMod->GetLength = internalInMod->GetLength;
				if (dllInMod->GetOutputTime == NULL)
					dllInMod->GetOutputTime = internalInMod->GetOutputTime;
				if (dllInMod->SetOutputTime == NULL)
					dllInMod->SetOutputTime = internalInMod->SetOutputTime;
				if (dllInMod->SetVolume == NULL)
					dllInMod->SetVolume = internalInMod->SetVolume;
				if (dllInMod->SetPan == NULL)
					dllInMod->SetPan = internalInMod->SetPan;
				if (dllInMod->SAVSAInit == NULL)
					dllInMod->SAVSAInit = internalInMod->SAVSAInit;
				if (dllInMod->SAVSADeInit == NULL)
					dllInMod->SAVSADeInit = internalInMod->SAVSADeInit;
				if (dllInMod->SAAddPCMData == NULL)
					dllInMod->SAAddPCMData = internalInMod->SAAddPCMData;
				if (dllInMod->SAGetMode == NULL)
					dllInMod->SAGetMode = internalInMod->SAGetMode;
				if (dllInMod->SAAdd == NULL)
					dllInMod->SAAdd = internalInMod->SAAdd;
				if (dllInMod->VSAAddPCMData == NULL)
					dllInMod->VSAAddPCMData = internalInMod->VSAAddPCMData;
				if (dllInMod->VSAGetMode == NULL)
					dllInMod->VSAGetMode = internalInMod->VSAGetMode;
				if (dllInMod->VSAAdd == NULL)
					dllInMod->VSAAdd = internalInMod->VSAAdd;
				if (dllInMod->VSASetInfo == NULL)
					dllInMod->VSASetInfo = internalInMod->VSASetInfo;
				if (dllInMod->dsp_isactive == NULL)
					dllInMod->dsp_isactive = internalInMod->dsp_isactive;
				if (dllInMod->dsp_dosamples == NULL)
					dllInMod->dsp_dosamples = internalInMod->dsp_dosamples;
				if (dllInMod->EQSet == NULL)
					dllInMod->EQSet = internalInMod->EQSet;
				if (dllInMod->SetInfo == NULL)
					dllInMod->SetInfo = internalInMod->SetInfo;
			}
			catch (...)
			{
				AfxMessageBox(m_HelperIn.MyLoadString(IDS_FAILED_LOAD_INPUT) + GetFileName(), MB_OK | MB_ICONEXCLAMATION);
				dllInMod = internalInMod;
				SetFilePath("STANDARD");
				hModIn = NULL;
			}
		}
	}
	else
	{
		dllInMod = internalInMod;
		SetFilePath("STANDARD");
		hModIn = NULL;
	}
	dllInMod->hDllInstance = hModIn;
	playerStatut = PLAYER_MODE_CLOSE;
}

void CDllInput::Config(HWND hwndParent) // configuration dialog
{
	try
	{
		dllInMod->Config(hwndParent);
	}
	catch (...)
	{
	}
}

void CDllInput::About(HWND hwndParent)  // about dialog
{
	try
	{
		dllInMod->About(hwndParent);
	}
	catch (...)
	{
	}
}

void CDllInput::Init()  		   // called at program init
{
	try
	{
		dllInMod->Init();
	}
	catch (...)
	{
		TRACE(_T("Problme d'init"));
	}
}

void CDllInput::Quit()  		   // called at program quit
{
	try
	{
		dllInMod->Quit();
	}
	catch (...)
	{
	}
}

void CDllInput::GetFileInfo(TCHAR* file, TCHAR* title, int* length_in_ms) // if file == NULL, current playing is used
{
	try
	{
		if ((dllInMod->version & IN_UNICODE) != IN_UNICODE)
		{
			static char ctitle[255];
			Strcpy(ctitle, 255, title); 
			static char filename[255];
			Strcpy(filename, 255, file); 
			dllInMod->GetFileInfo((wchar_t*)filename, (wchar_t*)ctitle, length_in_ms);
		}
		else
		{
			dllInMod->GetFileInfo(file, title, length_in_ms);
		}
	}
	catch (...)
	{
	}
}

int CDllInput::InfoBox(TCHAR* file, HWND hwndParent)
{
	int res = 0;
	try
	{
		if ((dllInMod->version & IN_UNICODE) != IN_UNICODE)
		{
			static char filename[255];
			Strcpy(filename, 255, file); 
			res = dllInMod->InfoBox((wchar_t*)filename, hwndParent);
		}
		else
		{
			res = dllInMod->InfoBox(file, hwndParent);
		}
	}
	catch (...)
	{
	}
	return res;
}

int CDllInput::IsOurFile(TCHAR* fn) // called before extension checks, to allow detection of mms://, etc
{
	int res = 0;
	try
	{
		if ((dllInMod->version & IN_UNICODE) != IN_UNICODE)
		{
			static char filename[255];
			Strcpy(filename, 255, fn); 
			res = dllInMod->IsOurFile((wchar_t*)filename);
		}
		else
		{
			res = dllInMod->IsOurFile(fn);
		}
	}
	catch (...)
	{
	}
	return res;
}

// playback stuff
int CDllInput::Play(TCHAR* fn)    // return zero on success, -1 on file-not-found, some other value on other (stopping winamp) error
{
	int res = 0;
	try
	{
		if ((dllInMod->version & IN_UNICODE) != IN_UNICODE)
		{
			static char filename[255];
			Strcpy(filename, 255, fn); 
			res = dllInMod->Play((wchar_t*)filename);
		}
		else
		{
			res = dllInMod->Play(fn);
		}
		if (res == 0)
			playerStatut = PLAYER_MODE_PLAYING;
		else
			playerStatut = PLAYER_MODE_CLOSE;
	}
	catch (...)
	{
	}
	return res;
}

void CDllInput::Pause() 	// pause stream
{
	dllInMod->Pause();
	playerStatut = PLAYER_MODE_PAUSED;
}

void CDllInput::UnPause()     // unpause stream
{
	dllInMod->UnPause();
	playerStatut = PLAYER_MODE_PLAYING;
}

int CDllInput::IsPaused()      // ispaused? return 1 if paused, 0 if not
{
	return dllInMod->IsPaused();
}

void CDllInput::Stop()  	  // stop (unload) stream
{
	try
	{
		dllInMod->Stop();
	}
	catch (...)
	{
	}
	playerStatut = PLAYER_MODE_CLOSE;
}
// time stuff
int CDllInput::GetLength()     // get length in ms
{
	int res = 0;
	try
	{
		res = dllInMod->GetLength();
	}
	catch (...)
	{
	}
	return res;
}

int CDllInput::GetOutputTime()   // returns current output time in ms. (usually returns outMod->GetOutputTime()
{
	int res = 0;
	try
	{
		res = dllInMod->GetOutputTime();
	}
	catch (...)
	{
	}
	return res;
}

void CDllInput::SetOutputTime(int time_in_ms) // seeks to point in stream (in ms). Usually you signal yoru thread to seek, which seeks and calls outMod->Flush()..
{
	try
	{
		dllInMod->SetOutputTime(time_in_ms);
	}
	catch (...)
	{
	}
}

// volume stuff
void CDllInput::SetVolume(int volume) // from 0 to 255.. usually just call outMod->SetVolume
{
	try
	{
		dllInMod->SetVolume(volume);
	}
	catch (...)
	{
	}
}

void CDllInput::SetPan(int pan) // from -127 to 127.. usually just call outMod->SetPan
{
	try
	{
		dllInMod->SetPan(pan);
	}
	catch (...)
	{
	}
}

// info setting (filled in by winamp)
void CDllInput::SetInfo(int bitrate, int srate, int stereo, int synched) // if -1, changes ignored? :)
{
	try
	{
		dllInMod->SetInfo(bitrate, srate, stereo, synched);
	}
	catch (...)
	{
	}
}

int CDllInput::GetVersion() 	   // module type (IN_VER)
{
	int res = 0;
	try
	{
		res = dllInMod->version;
	}
	catch (...)
	{
	}
	return res;
}
char* CDllInput::GetDescription()     // description of module, with version string
{
	try
	{
		return dllInMod->description;
	}
	catch (...)
	{
	}
}

void CDllInput::SethMainWindow(HWND hWnd)   // winamp's main window (filled in by winamp)
{
	dllInMod->hMainWindow = hWnd;
	internalIn->SethMainWindow(hWnd);
}
void CDllInput::SethDllInstance(HINSTANCE hDLL)   // DLL instance handle (Also filled in by winamp)
{
	hModIn = hDLL;
}

char* CDllInput::GetFileExtensions()   // "mp3\0Layer 3 MPEG\0mp2\0Layer 2 MPEG\0mpg\0Layer 1 MPEG\0"
{
	static char resExt[255];
	resExt[0] = 0;
	char* extensions = dllInMod->FileExtensions;
	CString ext = "";
	CString extDesc = "";
	int startExtDesc = 0;
	int extSize = 0;
	int extDescSize = 0;

	ext = extensions;
	extSize = ext.GetLength();
	startExtDesc = ext.GetLength() + 1;
	extDesc = extensions + startExtDesc;
	extDescSize = extDesc.GetLength();
	do
	{
		strcat(resExt, extensions);
		strcat(resExt, ";");
		extensions += extSize + extDescSize + 2;
		ext = extensions;
		extSize = ext.GetLength();
		startExtDesc = ext.GetLength() + 1;
		extDesc = extensions + startExtDesc;
		extDescSize = extDesc.GetLength();
	}
	while (ext != "");
	return resExt;
}

int CDllInput::isSeekable() 	 // is this stream seekable?
{
	int res = 0;
	try
	{
		res = dllInMod->is_seekable;
	}
	catch (...)
	{
	}
	return res;
}

int CDllInput::isUsesOutputPlug()      // does this plug-in use the output plug-ins? (musn't ever change, ever :)
{
	int res = 0;
	try
	{
		res = dllInMod->UsesOutputPlug;
	}
	catch (...)
	{
	}
	return res;
}

void CDllInput::SetOutMod(Out_Module* out) // filled in by winamp, optionally used :)
{
	dllInMod->outMod = out;
}

In_Module* CDllInput::GetIn_Module()
{
	return dllInMod;
}

int CDllInput::GetPlayerMode()
{
	return playerStatut;
}

void CDllInput::SetPlayerMode(int statut)
{
	playerStatut = statut;
}

int CDllInput::winampGetExtendedFileInfo(extendedFileInfoStruct ExtendedFileInfo)
{
	int res = 0;
	ExtendedFileInfo.ret[0] = 0;
	try
	{
		if (pfun2 != NULL)
		{
			res = (pfun2)(ExtendedFileInfo);
		}
		else
		{
			if (internalIn->IsOurFile((TCHAR *)ExtendedFileInfo.filename) == TRUE)
			{
				res = internalIn->winampGetExtendedFileInfo(ExtendedFileInfo);
			}
			else
			{
				char* pExtToken = strrev(strlwr(strdup(internalIn->GetFileExtensions())));
				char* extToken = strtok(pExtToken, ";");
				char* cFn = strrev(strlwr(strdup(ExtendedFileInfo.filename)));
				while (extToken != NULL)
				{
					if (strstr(cFn, extToken) == cFn)
					{
						res = internalIn->winampGetExtendedFileInfo(ExtendedFileInfo);
						break;
					}
					extToken = strtok(NULL, ";");
				}
				delete pExtToken;
				delete cFn;
			}
		}
	}
	catch (...)
	{
	}
	return res;
}

int CDllInput::winampGetExtendedFileInfoW(extendedFileInfoStructW ExtendedFileInfo)
{
	int res = 0;
	ExtendedFileInfo.ret[0] = 0;
	try
	{
		if (pfun3 != NULL)
		{
			res = (pfun3)(ExtendedFileInfo);
		}
		else
		{
			if (internalIn->IsOurFile((TCHAR *)ExtendedFileInfo.filename) == TRUE)
			{
				res = internalIn->winampGetExtendedFileInfoW(ExtendedFileInfo);
			}
			else
			{
				WCHAR extension[5000];

				Strcpy(extension, 5000, internalIn->GetFileExtensions(), 5000);
				WCHAR* pExtToken = wcsrev(wcslwr(wcsdup(extension)));
				WCHAR* extToken = wcstok(pExtToken, L";");
				WCHAR* cFn = wcsrev(wcslwr(wcsdup(ExtendedFileInfo.filename)));
				while (extToken != NULL)
				{
					if (wcsstr(cFn, extToken) == cFn)
					{
						res = internalIn->winampGetExtendedFileInfoW(ExtendedFileInfo);
						break;
					}
					extToken = wcstok(NULL, L";");
				}
				delete pExtToken;
				delete cFn;
			}
		}
	}
	catch (...)
	{
	}
	return res;
}